@@ -17,6 +17,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import functools
 from oslo.config import cfg
 
 from glance.common import crypt
@@ -25,31 +26,20 @@ import glance.domain
 import glance.domain.proxy
 from glance.openstack.common import importutils
 
-sql_connection_opt = cfg.StrOpt('sql_connection',
-                                default='sqlite:///glance.sqlite',
-                                secret=True,
-                                metavar='CONNECTION',
-                                help='A valid SQLAlchemy connection '
-                                     'string for the registry database. '
-                                     'Default: %(default)s')
+db_opt = cfg.BoolOpt('use_tpool',
+                     default=False,
+                     help='Enable the use of thread pooling for '
+                     'all DB API calls')
 
 CONF = cfg.CONF
-CONF.register_opt(sql_connection_opt)
 CONF.import_opt('metadata_encryption_key', 'glance.common.config')
-
-
-def add_cli_options():
-    """
-    Adds any configuration options that the db layer might have.
-
-    :retval None
-    """
-    CONF.unregister_opt(sql_connection_opt)
-    CONF.register_cli_opt(sql_connection_opt)
+CONF.register_opt(db_opt)
 
 
 def get_api():
-    return importutils.import_module(CONF.data_api)
+    if not CONF.use_tpool:
+        return importutils.import_module(CONF.data_api)
+    return ThreadPoolWrapper(CONF.data_api)
 
 
 # attributes common to all models
@@ -104,7 +94,11 @@ class ImageRepo(object):
         locations = db_image['locations']
         if CONF.metadata_encryption_key:
             key = CONF.metadata_encryption_key
-            locations = [crypt.urlsafe_decrypt(key, l) for l in locations]
+            ld = []
+            for l in locations:
+                url = crypt.urlsafe_decrypt(key, l['url'])
+                ld.append({'url': url, 'metadata': l['metadata']})
+            locations = ld
         return glance.domain.Image(
             image_id=db_image['id'],
             name=db_image['name'],
@@ -129,7 +123,11 @@ class ImageRepo(object):
         locations = image.locations
         if CONF.metadata_encryption_key:
             key = CONF.metadata_encryption_key
-            locations = [crypt.urlsafe_encrypt(key, l) for l in locations]
+            ld = []
+            for l in locations:
+                url = crypt.urlsafe_encrypt(key, l['url'])
+                ld.append({'url': url, 'metadata': l['metadata']})
+            locations = ld
         return {
             'id': image.image_id,
             'name': image.name,
@@ -231,6 +229,17 @@ class ImageMemberRepo(object):
         return image_members
 
     def add(self, image_member):
+        try:
+            self.get(image_member.member_id)
+        except exception.NotFound:
+            pass
+        else:
+            msg = _('The target member %(member_id)s is already '
+                    'associated with image %(image_id)s.' %
+                    dict(member_id=image_member.member_id,
+                         image_id=self.image.image_id))
+            raise exception.Duplicate(msg)
+
         image_member_values = self._format_image_member_to_db(image_member)
         new_values = self.db_api.image_member_create(self.context,
                                                      image_member_values)
@@ -262,7 +271,7 @@ class ImageMemberRepo(object):
                                                         self.context,
                                                         self.image.image_id,
                                                         member_id)
-            if len(db_api_image_member) == 0:
+            if not db_api_image_member:
                 raise exception.NotFound()
         except (exception.NotFound, exception.Forbidden):
             raise exception.NotFound()
@@ -270,3 +279,21 @@ class ImageMemberRepo(object):
         image_member = self._format_image_member_from_db(
                                                     db_api_image_member[0])
         return image_member
+
+
+class ThreadPoolWrapper(object):
+
+    def __init__(self, wrapped):
+        self.wrapped = importutils.import_module(wrapped)
+
+    def __getattr__(self, key):
+        original = getattr(self.wrapped, key)
+        if not callable(original):
+            return original
+
+        @functools.wraps(original)
+        def wrapper(*args, **kwargs):
+            from eventlet import tpool
+            output = tpool.execute(original, *args, **kwargs)
+            return output
+        return wrapper
