@@ -49,6 +49,14 @@ LOG = os_logging.getLogger(__name__)
 STATUSES = ['active', 'saving', 'queued', 'killed', 'pending_delete',
             'deleted']
 
+sql_connection_opt = cfg.StrOpt('sql_connection',
+                                default='sqlite:///glance.sqlite',
+                                secret=True,
+                                metavar='CONNECTION',
+                                help=_('A valid SQLAlchemy connection '
+                                       'string for the registry database. '
+                                       'Default: %(default)s'))
+
 db_opts = [
     cfg.IntOpt('sql_idle_timeout', default=3600,
                help=_('Period in seconds after which SQLAlchemy should '
@@ -62,14 +70,26 @@ db_opts = [
     cfg.BoolOpt('db_auto_create', default=False,
                 help=_('A boolean that determines if the database will be '
                        'automatically created.')),
+    cfg.BoolOpt('sqlalchemy_debug', default=False,
+                help=_('Enable debug logging in sqlalchemy which prints '
+                       'every query and result'))
 ]
 
 CONF = cfg.CONF
+CONF.register_opt(sql_connection_opt)
 CONF.register_opts(db_opts)
 CONF.import_opt('debug', 'glance.openstack.common.log')
 
 
-def ping_listener(dbapi_conn, connection_rec, connection_proxy):
+def add_cli_options():
+    """Allows passing sql_connection as a CLI argument."""
+
+    # NOTE(flaper87): Find a better place / way for this.
+    CONF.unregister_opt(sql_connection_opt)
+    CONF.register_cli_opt(sql_connection_opt)
+
+
+def _ping_listener(dbapi_conn, connection_rec, connection_proxy):
 
     """
     Ensures that MySQL connections checked out of the
@@ -81,7 +101,7 @@ def ping_listener(dbapi_conn, connection_rec, connection_proxy):
 
     try:
         dbapi_conn.cursor().execute('select 1')
-    except dbapi_conn.OperationalError, ex:
+    except dbapi_conn.OperationalError as ex:
         if ex.args[0] in (2006, 2013, 2014, 2045, 2055):
             msg = 'Got mysql server has gone away: %s' % ex
             LOG.warn(msg)
@@ -92,7 +112,7 @@ def ping_listener(dbapi_conn, connection_rec, connection_proxy):
 
 def setup_db_env():
     """
-    Setup configuration for database
+    Setup global configuration for database.
     """
     global sa_logger, _IDLE_TIMEOUT, _MAX_RETRIES, _RETRY_INTERVAL, _CONNECTION
 
@@ -101,20 +121,22 @@ def setup_db_env():
     _RETRY_INTERVAL = CONF.sql_retry_interval
     _CONNECTION = CONF.sql_connection
     sa_logger = logging.getLogger('sqlalchemy.engine')
-    if CONF.debug:
+    if CONF.sqlalchemy_debug:
         sa_logger.setLevel(logging.DEBUG)
 
 
-def configure_db():
+def clear_db_env():
     """
-    Establish the database, create an engine if needed, and
-    register the models.
+    Unset global configuration variables for database.
     """
-    setup_db_env()
-    get_engine()
+    global _ENGINE, _MAKER, _MAX_RETRIES, _RETRY_INTERVAL, _CONNECTION
+    _ENGINE = None
+    _MAKER = None
+    _MAX_RETRIES = None
+    _RETRY_INTERVAL = None
 
 
-def check_mutate_authorization(context, image_ref):
+def _check_mutate_authorization(context, image_ref):
     if not is_image_mutable(context, image_ref):
         LOG.info(_("Attempted to modify image user did not own."))
         msg = _("You do not own this image")
@@ -126,12 +148,12 @@ def check_mutate_authorization(context, image_ref):
         raise exc_class(msg)
 
 
-def get_session(autocommit=True, expire_on_commit=False):
+def _get_session(autocommit=True, expire_on_commit=False):
     """Helper method to grab session"""
     global _MAKER
     if not _MAKER:
         get_engine()
-        get_maker(autocommit, expire_on_commit)
+        _get_maker(autocommit, expire_on_commit)
         assert(_MAKER)
     session = _MAKER()
     return session
@@ -158,18 +180,18 @@ def get_engine():
             _ENGINE = sqlalchemy.create_engine(_CONNECTION, **engine_args)
 
             if 'mysql' in connection_dict.drivername:
-                sqlalchemy.event.listen(_ENGINE, 'checkout', ping_listener)
+                sqlalchemy.event.listen(_ENGINE, 'checkout', _ping_listener)
 
-            _ENGINE.connect = wrap_db_error(_ENGINE.connect)
+            _ENGINE.connect = _wrap_db_error(_ENGINE.connect)
             _ENGINE.connect()
-        except Exception, err:
+        except Exception as err:
             msg = _("Error configuring registry database with supplied "
                     "sql_connection. Got error: %s") % err
             LOG.error(msg)
             raise
 
         sa_logger = logging.getLogger('sqlalchemy.engine')
-        if CONF.debug:
+        if CONF.sqlalchemy_debug:
             sa_logger.setLevel(logging.DEBUG)
 
         if CONF.db_auto_create:
@@ -186,7 +208,7 @@ def get_engine():
     return _ENGINE
 
 
-def get_maker(autocommit=True, expire_on_commit=False):
+def _get_maker(autocommit=True, expire_on_commit=False):
     """Return a SQLAlchemy sessionmaker."""
     """May assign __MAKER if not already assigned"""
     global _MAKER, _ENGINE
@@ -198,7 +220,7 @@ def get_maker(autocommit=True, expire_on_commit=False):
     return _MAKER
 
 
-def is_db_connection_error(args):
+def _is_db_connection_error(args):
     """Return True if error in connecting to db."""
     # NOTE(adam_g): This is currently MySQL specific and needs to be extended
     #               to support Postgres and others.
@@ -209,13 +231,13 @@ def is_db_connection_error(args):
     return False
 
 
-def wrap_db_error(f):
+def _wrap_db_error(f):
     """Retry DB connection. Copied from nova and modified."""
     def _wrap(*args, **kwargs):
         try:
             return f(*args, **kwargs)
-        except sqlalchemy.exc.OperationalError, e:
-            if not is_db_connection_error(e.args[0]):
+        except sqlalchemy.exc.OperationalError as e:
+            if not _is_db_connection_error(e.args[0]):
                 raise
 
             remaining_attempts = _MAX_RETRIES
@@ -226,9 +248,9 @@ def wrap_db_error(f):
                 time.sleep(_RETRY_INTERVAL)
                 try:
                     return f(*args, **kwargs)
-                except sqlalchemy.exc.OperationalError, e:
+                except sqlalchemy.exc.OperationalError as e:
                     if (remaining_attempts == 0 or
-                        not is_db_connection_error(e.args[0])):
+                            not _is_db_connection_error(e.args[0])):
                         raise
                 except sqlalchemy.exc.DBAPIError:
                     raise
@@ -254,30 +276,33 @@ def image_update(context, image_id, values, purge_props=False):
 
 def image_destroy(context, image_id):
     """Destroy the image or raise if it does not exist."""
-    session = get_session()
+    session = _get_session()
     with session.begin():
         image_ref = _image_get(context, image_id, session=session)
 
         # Perform authorization check
-        check_mutate_authorization(context, image_ref)
-
-        _image_locations_set(image_ref.id, [], session)
+        _check_mutate_authorization(context, image_ref)
 
         image_ref.delete(session=session)
+        delete_time = image_ref.deleted_at
 
-        for prop_ref in image_ref.properties:
-            image_property_delete(context, prop_ref, session=session)
+        _image_locations_delete_all(context, image_ref.id, delete_time,
+                                    session)
 
-        members = _image_member_find(context, session, image_id=image_id)
-        for memb_ref in members:
-            _image_member_delete(context, memb_ref, session)
+        _image_property_delete_all(context, image_id, delete_time, session)
+
+        _image_member_delete_all(context, image_id, delete_time, session)
+
+        _image_tag_delete_all(context, image_id, delete_time, session)
 
     return _normalize_locations(image_ref)
 
 
 def _normalize_locations(image):
     undeleted_locations = filter(lambda x: not x.deleted, image['locations'])
-    image['locations'] = [loc['value'] for loc in undeleted_locations]
+    image['locations'] = [{'url': loc['value'],
+                           'metadata': loc['meta_data']}
+                          for loc in undeleted_locations]
     return image
 
 
@@ -290,7 +315,7 @@ def image_get(context, image_id, session=None, force_show_deleted=False):
 
 def _image_get(context, image_id, session=None, force_show_deleted=False):
     """Get an image or raise if it does not exist."""
-    session = session or get_session()
+    session = session or _get_session()
 
     try:
         query = session.query(models.Image)\
@@ -305,11 +330,15 @@ def _image_get(context, image_id, session=None, force_show_deleted=False):
         image = query.one()
 
     except sa_orm.exc.NoResultFound:
-        raise exception.NotFound("No image found with ID %s" % image_id)
+        msg = (_("No image found with ID %s") % image_id)
+        LOG.debug(msg)
+        raise exception.NotFound(msg)
 
     # Make sure they can look at it
     if not is_image_visible(context, image):
-        raise exception.Forbidden("Image not visible to you")
+        msg = (_("Forbidding request, image %s not visible") % image_id)
+        LOG.debug(msg)
+        raise exception.Forbidden(msg)
 
     return image
 
@@ -393,8 +422,8 @@ def is_image_visible(context, image, status=None):
     return False
 
 
-def paginate_query(query, model, limit, sort_keys, marker=None,
-                   sort_dir=None, sort_dirs=None):
+def _paginate_query(query, model, limit, sort_keys, marker=None,
+                    sort_dir=None, sort_dirs=None):
     """Returns a query with sorting / pagination criteria added.
 
     Pagination works by requiring a unique sort_key, specified by sort_keys.
@@ -455,11 +484,15 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
             raise exception.InvalidSortKey()
         query = query.order_by(sort_dir_func(sort_key_attr))
 
+    default = ''  # Default to an empty string if NULL
+
     # Add pagination
     if marker is not None:
         marker_values = []
         for sort_key in sort_keys:
             v = getattr(marker, sort_key)
+            if v is None:
+                v = default
             marker_values.append(v)
 
         # Build up an array of sort criteria as in the docstring
@@ -468,13 +501,19 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
             crit_attrs = []
             for j in xrange(0, i):
                 model_attr = getattr(model, sort_keys[j])
-                crit_attrs.append((model_attr == marker_values[j]))
+                attr = sa_sql.expression.case([(model_attr != None,
+                                              model_attr), ],
+                                              else_=default)
+                crit_attrs.append((attr == marker_values[j]))
 
             model_attr = getattr(model, sort_keys[i])
+            attr = sa_sql.expression.case([(model_attr != None,
+                                          model_attr), ],
+                                          else_=default)
             if sort_dirs[i] == 'desc':
-                crit_attrs.append((model_attr < marker_values[i]))
+                crit_attrs.append((attr < marker_values[i]))
             elif sort_dirs[i] == 'asc':
-                crit_attrs.append((model_attr > marker_values[i]))
+                crit_attrs.append((attr > marker_values[i]))
             else:
                 raise ValueError(_("Unknown sort direction, "
                                    "must be 'desc' or 'asc'"))
@@ -493,7 +532,8 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
 
 def image_get_all(context, filters=None, marker=None, limit=None,
                   sort_key='created_at', sort_dir='desc',
-                  member_status='accepted'):
+                  member_status='accepted', is_public=None,
+                  admin_as_user=False):
     """
     Get all images that match zero or more filters.
 
@@ -504,54 +544,73 @@ def image_get_all(context, filters=None, marker=None, limit=None,
     :param limit: maximum number of images to return
     :param sort_key: image attribute by which results should be sorted
     :param sort_dir: direction in which results should be sorted (asc, desc)
+    :param member_status: only return shared images that have this membership
+                          status
+    :param is_public: If true, return only public images. If false, return
+                      only private and shared images.
+    :param admin_as_user: For backwards compatibility. If true, then return to
+                      an admin the equivalent set of images which it would see
+                      if it were a regular user
     """
     filters = filters or {}
 
-    session = get_session()
-    query = session.query(models.Image)\
-                   .options(sa_orm.joinedload(models.Image.properties))\
-                   .options(sa_orm.joinedload(models.Image.locations))
+    session = _get_session()
+    query_image = session.query(models.Image)
+    query_member = session.query(models.Image).join(models.Image.members)
 
-    # NOTE(markwash) treat is_public=None as if it weren't filtered
-    if 'is_public' in filters and filters['is_public'] is None:
-        del filters['is_public']
-
-    if not context.is_admin:
+    if (not context.is_admin) or admin_as_user == True:
         visibility_filters = [models.Image.is_public == True]
+        member_filters = [models.ImageMember.deleted == False]
 
         if context.owner is not None:
             if member_status == 'all':
                 visibility_filters.extend([
-                    models.Image.owner == context.owner,
-                    models.Image.members.any(member=context.owner,
-                                             deleted=False),
-                ])
+                    models.Image.owner == context.owner])
+                member_filters.extend([
+                    models.ImageMember.member == context.owner])
             else:
                 visibility_filters.extend([
-                    models.Image.owner == context.owner,
-                    models.Image.members.any(member=context.owner,
-                                             deleted=False,
-                                             status=member_status),
-                ])
+                    models.Image.owner == context.owner])
+                member_filters.extend([
+                    models.ImageMember.member == context.owner,
+                    models.ImageMember.status == member_status])
 
-        query = query.filter(sa_sql.or_(*visibility_filters))
+        query_image = query_image.filter(sa_sql.or_(*visibility_filters))
+        query_member = query_member.filter(sa_sql.and_(*member_filters))
+
+    query = query_image.union(query_member)
 
     if 'visibility' in filters:
         visibility = filters.pop('visibility')
         if visibility == 'public':
             query = query.filter(models.Image.is_public == True)
-            filters['is_public'] = True
         elif visibility == 'private':
-            filters['is_public'] = False
-            if (not context.is_admin) and context.owner is not None:
+            query = query.filter(models.Image.is_public == False)
+            if context.owner is not None and ((not context.is_admin)
+                                              or admin_as_user == True):
                 query = query.filter(
-                            models.Image.owner == context.owner)
+                    models.Image.owner == context.owner)
         else:
-            query = query.filter(
-                        models.Image.members.any(member=context.owner,
-                                                 deleted=False))
+            query_member = query_member.filter(
+                models.ImageMember.member == context.owner,
+                models.ImageMember.deleted == False)
+            query = query_member
+
+    if is_public is not None:
+        query = query.filter(models.Image.is_public == is_public)
+
+    if 'is_public' in filters:
+        spec = models.Image.properties.any(name='is_public',
+                                           value=filters.pop('is_public'),
+                                           deleted=False)
+        query = query.filter(spec)
 
     showing_deleted = False
+
+    if 'checksum' in filters:
+        checksum = filters.pop('checksum')
+        query = query.filter(models.Image.checksum == checksum)
+
     if 'changes-since' in filters:
         # normalize timestamp to UTC, as sqlalchemy doesn't appear to
         # respect timezone offsets
@@ -572,6 +631,12 @@ def image_get_all(context, filters=None, marker=None, limit=None,
                                                          value=v,
                                                          deleted=False))
 
+    if 'tags' in filters:
+        tags = filters.pop('tags')
+        for tag in tags:
+            query = query.filter(models.Image.tags.any(value=tag,
+                                                       deleted=False))
+
     for (k, v) in filters.items():
         if v is not None:
             key = k
@@ -599,10 +664,16 @@ def image_get_all(context, filters=None, marker=None, limit=None,
         marker_image = _image_get(context, marker,
                                   force_show_deleted=showing_deleted)
 
-    query = paginate_query(query, models.Image, limit,
-                           [sort_key, 'created_at', 'id'],
-                           marker=marker_image,
-                           sort_dir=sort_dir)
+    sort_keys = ['created_at', 'id']
+    sort_keys.insert(0, sort_key) if sort_key not in sort_keys else sort_keys
+
+    query = _paginate_query(query, models.Image, limit,
+                            sort_keys,
+                            marker=marker_image,
+                            sort_dir=sort_dir)
+
+    query = query.options(sa_orm.joinedload(models.Image.properties))\
+                 .options(sa_orm.joinedload(models.Image.locations))
 
     return [_normalize_locations(image.to_dict()) for image in query.all()]
 
@@ -617,14 +688,31 @@ def _drop_protected_attrs(model_class, values):
             del values[attr]
 
 
-def validate_image(values):
+def _image_get_disk_usage_by_owner(owner, session, image_id=None):
+    query = session.query(models.Image)
+    query = query.filter(models.Image.owner == owner)
+    if image_id is not None:
+        query = query.filter(models.Image.id != image_id)
+    query = query.filter(models.Image.size > 0)
+    query = query.filter(~models.Image.status.in_(['killed',
+                                                   'pending_delete',
+                                                   'deleted']))
+    images = query.all()
+
+    total = 0
+    for i in images:
+        locations = [l for l in i.locations if not l['deleted']]
+        total += (i.size * len(locations))
+    return total
+
+
+def _validate_image(values):
     """
     Validates the incoming data and raises a Invalid exception
     if anything is out of order.
 
     :param values: Mapping of image metadata to check
     """
-    status = values.get('status')
 
     status = values.get('status', None)
     if not status:
@@ -652,7 +740,11 @@ def _image_update(context, values, image_id, purge_props=False):
     :param values: A dict of attributes to set
     :param image_id: If None, create the image, otherwise, find and update it
     """
-    session = get_session()
+
+    #NOTE(jbresnah) values is altered in this so a copy is needed
+    values = values.copy()
+
+    session = _get_session()
     with session.begin():
 
         # Remove the properties passed in the values mapping. We
@@ -663,17 +755,13 @@ def _image_update(context, values, image_id, purge_props=False):
         # not a dict.
         properties = values.pop('properties', {})
 
-        try:
-            locations = values.pop('locations')
-            locations_provided = True
-        except KeyError:
-            locations_provided = False
+        location_data = values.pop('locations', None)
 
         if image_id:
             image_ref = _image_get(context, image_id, session=session)
 
             # Perform authorization check
-            check_mutate_authorization(context, image_ref)
+            _check_mutate_authorization(context, image_ref)
         else:
             if values.get('size') is not None:
                 values['size'] = int(values['size'])
@@ -704,7 +792,7 @@ def _image_update(context, values, image_id, purge_props=False):
         # investigation, the @validates decorator does not validate
         # on new records, only on existing records, which is, well,
         # idiotic.
-        values = validate_image(image_ref.to_dict())
+        values = _validate_image(image_ref.to_dict())
         _update_values(image_ref, values)
 
         try:
@@ -716,8 +804,8 @@ def _image_update(context, values, image_id, purge_props=False):
         _set_properties_for_image(context, image_ref, properties, purge_props,
                                   session)
 
-    if locations_provided:
-        _image_locations_set(image_ref.id, locations, session)
+    if location_data is not None:
+        _image_locations_set(image_ref.id, location_data, session)
 
     return image_get(context, image_ref.id)
 
@@ -731,10 +819,22 @@ def _image_locations_set(image_id, locations, session):
         location_ref.delete(session=session)
 
     for location in locations:
-        location_ref = models.ImageLocation(image_id=image_id, value=location)
+        location_ref = models.ImageLocation(image_id=image_id,
+                                            value=location['url'],
+                                            meta_data=location['metadata'])
         location_ref.save()
 
 
+def _image_locations_delete_all(context, image_id, delete_time=None,
+                                session=None):
+    """Delete all image locations for given image"""
+    locs_updated_count = _image_child_entry_delete_all(models.ImageLocation,
+                                                       image_id,
+                                                       delete_time,
+                                                       session)
+    return locs_updated_count
+
+
 def _set_properties_for_image(context, image_ref, properties,
                               purge_props=False, session=None):
     """
@@ -764,13 +864,47 @@ def _set_properties_for_image(context, image_ref, properties,
         for key in orig_properties.keys():
             if key not in properties:
                 prop_ref = orig_properties[key]
-                image_property_delete(context, prop_ref, session=session)
+                image_property_delete(context, prop_ref.name,
+                                      image_ref.id, session=session)
+
+
+def _image_child_entry_delete_all(child_model_cls, image_id, delete_time=None,
+                                  session=None):
+    """Deletes all the child entries for the given image id.
+
+    Deletes all the child entries of the given child entry ORM model class
+    using the parent image's id.
+
+    The child entry ORM model class can be one of the following:
+    model.ImageLocation, model.ImageProperty, model.ImageMember and
+    model.ImageTag.
+
+    :param child_model_cls: the ORM model class.
+    :param image_id: id of the image whose child entries are to be deleted.
+    :param delete_time: datetime of deletion to be set.
+                        If None, uses current datetime.
+    :param session: A SQLAlchemy session to use (if present)
+
+    :rtype: int
+    :return: The number of child entries got soft-deleted.
+    """
+    session = session or _get_session()
+
+    query = session.query(child_model_cls) \
+        .filter_by(image_id=image_id) \
+        .filter_by(deleted=False)
+
+    delete_time = delete_time or timeutils.utcnow()
+
+    count = query.update({"deleted": True, "deleted_at": delete_time})
+    return count
 
 
 def image_property_create(context, values, session=None):
     """Create an ImageProperty object"""
     prop_ref = models.ImageProperty()
-    return _image_property_update(context, prop_ref, values, session=session)
+    prop = _image_property_update(context, prop_ref, values, session=session)
+    return prop.to_dict()
 
 
 def _image_property_update(context, prop_ref, values, session=None):
@@ -784,12 +918,25 @@ def _image_property_update(context, prop_ref, values, session=None):
     return prop_ref
 
 
-def image_property_delete(context, prop_ref, session=None):
+def image_property_delete(context, prop_ref, image_ref, session=None):
     """
     Used internally by image_property_create and image_property_update
     """
-    prop_ref.delete(session=session)
-    return prop_ref
+    session = session or _get_session()
+    prop = session.query(models.ImageProperty).filter_by(image_id=image_ref,
+                                                         name=prop_ref).one()
+    prop.delete(session=session)
+    return prop
+
+
+def _image_property_delete_all(context, image_id, delete_time=None,
+                               session=None):
+    """Delete all image properties for given image"""
+    props_updated_count = _image_child_entry_delete_all(models.ImageProperty,
+                                                        image_id,
+                                                        delete_time,
+                                                        session)
+    return props_updated_count
 
 
 def image_member_create(context, values, session=None):
@@ -814,7 +961,7 @@ def _image_member_format(member_ref):
 
 def image_member_update(context, memb_id, values):
     """Update an ImageMember object"""
-    session = get_session()
+    session = _get_session()
     memb_ref = _image_member_get(context, memb_id, session)
     _image_member_update(context, memb_ref, values, session)
     return _image_member_format(memb_ref)
@@ -832,7 +979,7 @@ def _image_member_update(context, memb_ref, values, session=None):
 
 def image_member_delete(context, memb_id, session=None):
     """Delete an ImageMember object"""
-    session = session or get_session()
+    session = session or _get_session()
     member_ref = _image_member_get(context, memb_id, session)
     _image_member_delete(context, member_ref, session)
 
@@ -841,6 +988,16 @@ def _image_member_delete(context, memb_ref, session):
     memb_ref.delete(session=session)
 
 
+def _image_member_delete_all(context, image_id, delete_time=None,
+                             session=None):
+    """Delete all image members for given image"""
+    members_updated_count = _image_child_entry_delete_all(models.ImageMember,
+                                                          image_id,
+                                                          delete_time,
+                                                          session)
+    return members_updated_count
+
+
 def _image_member_get(context, memb_id, session):
     """Fetch an ImageMember entity by id"""
     query = session.query(models.ImageMember)
@@ -854,7 +1011,7 @@ def image_member_find(context, image_id=None, member=None, status=None):
     :param image_id: identifier of image entity
     :param member: tenant to which membership has been granted
     """
-    session = get_session()
+    session = _get_session()
     members = _image_member_find(context, session, image_id, member, status)
     return [_image_member_format(m) for m in members]
 
@@ -896,7 +1053,7 @@ def _can_show_deleted(context):
 
 
 def image_tag_set_all(context, image_id, tags):
-    session = get_session()
+    session = _get_session()
     existing_tags = set(image_tag_get_all(context, image_id, session))
     tags = set(tags)
 
@@ -914,7 +1071,7 @@ def image_tag_set_all(context, image_id, tags):
 
 def image_tag_create(context, image_id, value, session=None):
     """Create an image tag."""
-    session = session or get_session()
+    session = session or _get_session()
     tag_ref = models.ImageTag(image_id=image_id, value=value)
     tag_ref.save(session=session)
     return tag_ref['value']
@@ -922,7 +1079,7 @@ def image_tag_create(context, image_id, value, session=None):
 
 def image_tag_delete(context, image_id, value, session=None):
     """Delete an image tag."""
-    session = session or get_session()
+    session = session or _get_session()
     query = session.query(models.ImageTag)\
                    .filter_by(image_id=image_id)\
                    .filter_by(value=value)\
@@ -935,12 +1092,28 @@ def image_tag_delete(context, image_id, value, session=None):
     tag_ref.delete(session=session)
 
 
+def _image_tag_delete_all(context, image_id, delete_time=None, session=None):
+    """Delete all image tags for given image"""
+    tags_updated_count = _image_child_entry_delete_all(models.ImageTag,
+                                                       image_id,
+                                                       delete_time,
+                                                       session)
+    return tags_updated_count
+
+
 def image_tag_get_all(context, image_id, session=None):
     """Get a list of tags for a specific image."""
-    session = session or get_session()
+    session = session or _get_session()
     tags = session.query(models.ImageTag)\
                   .filter_by(image_id=image_id)\
                   .filter_by(deleted=False)\
                   .order_by(sqlalchemy.asc(models.ImageTag.created_at))\
                   .all()
     return [tag['value'] for tag in tags]
+
+
+def user_get_storage_usage(context, owner_id, image_id=None, session=None):
+    session = session or _get_session()
+    total_size = _image_get_disk_usage_by_owner(
+        owner_id, session, image_id=image_id)
+    return total_size
