@@ -13,11 +13,31 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
+from oslo.config import cfg
+
 from glance.common import exception
 from glance.openstack.common import timeutils
 from glance.openstack.common import uuidutils
 
 
+image_format_opts = [
+    cfg.ListOpt('container_formats',
+                default=['ami', 'ari', 'aki', 'bare', 'ovf'],
+                help=_("Supported values for the 'container_format' "
+                       "image attribute")),
+    cfg.ListOpt('disk_formats',
+                default=['ami', 'ari', 'aki', 'vhd', 'vmdk', 'raw', 'qcow2',
+                         'vdi', 'iso'],
+                help=_("Supported values for the 'disk_format' "
+                       "image attribute")),
+]
+
+
+CONF = cfg.CONF
+CONF.register_opts(image_format_opts)
+
+
 class ImageFactory(object):
     _readonly_properties = ['created_at', 'updated_at', 'status', 'checksum',
                             'size']
@@ -31,7 +51,7 @@ class ImageFactory(object):
                 raise exception.ReadonlyProperty(property=key)
 
     def _check_unexpected(self, kwargs):
-        if len(kwargs) > 0:
+        if kwargs:
             msg = 'new_image() got unexpected keywords %s'
             raise TypeError(msg % kwargs.keys())
 
@@ -82,9 +102,10 @@ class Image(object):
         self.disk_format = kwargs.pop('disk_format', None)
         self.container_format = kwargs.pop('container_format', None)
         self.size = kwargs.pop('size', None)
-        self.extra_properties = kwargs.pop('extra_properties', None) or {}
+        extra_properties = kwargs.pop('extra_properties', None) or {}
+        self.extra_properties = ExtraProperties(extra_properties)
         self.tags = kwargs.pop('tags', None) or []
-        if len(kwargs) > 0:
+        if kwargs:
             message = "__init__() got unexpected keyword argument '%s'"
             raise TypeError(message % kwargs.keys()[0])
 
@@ -137,6 +158,32 @@ class Image(object):
         raise NotImplementedError()
 
 
+class ExtraProperties(collections.MutableMapping, dict):
+
+    def __getitem__(self, key):
+        return dict.__getitem__(self, key)
+
+    def __setitem__(self, key, value):
+        return dict.__setitem__(self, key, value)
+
+    def __delitem__(self, key):
+        return dict.__delitem__(self, key)
+
+    def __eq__(self, other):
+        if isinstance(other, ExtraProperties):
+            return dict(self).__eq__(dict(other))
+        elif isinstance(other, dict):
+            return dict(self).__eq__(other)
+        else:
+            return False
+
+    def __len__(self):
+        return dict(self).__len__()
+
+    def keys(self):
+        return dict(self).keys()
+
+
 class ImageMembership(object):
 
     def __init__(self, image_id, member_id, created_at, updated_at,
